@@ -82,6 +82,10 @@ let bad_char i s =
   let msg = Printf.sprintf "invalid character '%c' at %d" s.[i] i in
   Parse_error (msg, s)
 
+let octal_notation s =
+  let msg = Printf.sprintf "octal notation disallowed" in
+  Parse_error (msg, s)
+
 let is_number base n = n >= 0 && n < base
 
 let parse_int base s i =
@@ -113,6 +117,11 @@ let expect_char s i c =
 
 let expect_end s i = if String.length s <= !i then () else raise (bad_char !i s)
 
+let reject_octal s i =
+  if !i + 1 < String.length s then
+    if s.[!i] == '0' && is_number 10 (int_of_char s.[!i + 1]) then
+      raise (octal_notation s)
+
 let hex_char_of_int = function
   | 0 -> '0'
   | 1 -> '1'
@@ -147,12 +156,16 @@ module V4 = struct
   (* parsing *)
 
   let parse_dotted_quad s i =
+    reject_octal s i;
     let a = parse_dec_int s i in
     expect_char s i '.';
+    reject_octal s i;
     let b = parse_dec_int s i in
     expect_char s i '.';
+    reject_octal s i;
     let c = parse_dec_int s i in
     expect_char s i '.';
+    reject_octal s i;
     let d = parse_dec_int s i in
     let valid a = a land 0xff <> a in
     if valid a then raise (Parse_error ("first octet out of bounds", s))
@@ -438,6 +451,18 @@ module V4 = struct
   let is_multicast i = Prefix.(mem i multicast)
 
   let is_private i = scope i <> Global
+
+  module Set = Set.Make (struct
+    type nonrec t = t
+
+    let compare (a : t) (b : t) = compare a b
+  end)
+
+  module Map = Map.Make (struct
+    type nonrec t = t
+
+    let compare (a : t) (b : t) = compare a b
+  end)
 end
 
 module B128 = struct
@@ -979,6 +1004,18 @@ module V6 = struct
   let is_multicast i = Prefix.(mem i multicast)
 
   let is_private i = scope i <> Global
+
+  module Set = Set.Make (struct
+    type nonrec t = t
+
+    let compare (a : t) (b : t) = compare a b
+  end)
+
+  module Map = Map.Make (struct
+    type nonrec t = t
+
+    let compare (a : t) (b : t) = compare a b
+  end)
 end
 
 type ('v4, 'v6) v4v6 = V4 of 'v4 | V6 of 'v6
@@ -992,6 +1029,18 @@ let compare a b =
   | V4 _, V6 _ -> -1
   | V6 _, V4 _ -> 1
 
+module Set = Set.Make (struct
+  type nonrec t = t
+
+  let compare (a : t) (b : t) = compare a b
+end)
+
+module Map = Map.Make (struct
+  type nonrec t = t
+
+  let compare (a : t) (b : t) = compare a b
+end)
+
 let to_string = function V4 x -> V4.to_string x | V6 x -> V6.to_string x
 
 let to_buffer buf = function
